package de.ctdo.crashtest.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

public class MainGui extends JFrame implements IGuiControl {
    private final JTextArea textWall = new JTextArea();
    private final JLabel countDown = new JLabel();
    private final JLabel extraField = new JLabel();
    private final JPanel lowerPanel = new JPanel();
    
    private final List<GuiEventListener> listenerList = new ArrayList<GuiEventListener>();

    public MainGui() {
        initGui();
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setVisible(true);
        setBounds(0,0, 1280, 1024);

        addKeyListener(new KeyAdapter() {
            @Override
            public void keyTyped(KeyEvent e) {
                char lastKey = e.getKeyChar();

                for(GuiEventListener listener: listenerList) {
                    listener.keyPress(lastKey);
                }
            }
        });

        addWindowStateListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                for(GuiEventListener listener: listenerList) {
                    listener.windowClosing();
                }
            }
        });
    }

//
//
//    private void initTimer() {
//        timer = new Timer(100, new ActionListener() {
//            @Override
//            public void actionPerformed(ActionEvent e) {
//                timerSecondsLast--;
//                setTimerText();
//
//                if(timerSecondsLast <= 0) {
//                    timerSecondsLast = 0;
//                    timer.stop();
//                }
//            }
//        });
//    }
//
    private void setCountDownText(final int tseconds) {
        int mins  = tseconds / 600;
        int secs  = tseconds % 600 / 10 ;
        int tsecs = tseconds % 9;
        countDown.setText(" " + mins + ":" + secs + "." + tsecs);
    }

    private void initGui() {
        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        container.add(textWall, BorderLayout.NORTH);
        container.add(lowerPanel, BorderLayout.SOUTH);

        lowerPanel.setLayout(new BorderLayout());
        lowerPanel.add(countDown, BorderLayout.WEST);
        lowerPanel.add(extraField, BorderLayout.EAST);

        Font wallFont = new Font("Arcade Interlaced", Font.PLAIN, 66);
        Font lowerFont = new Font("Arcade Interlaced", Font.PLAIN, 80);
        Color fontColor = new Color(0x00, 190, 100);
        
        textWall.setFont(wallFont);
        textWall.setForeground(fontColor);
        textWall.setBackground(Color.BLACK);
        textWall.setText("follow the white rabbit...");
        textWall.setLineWrap(true);
        textWall.setWrapStyleWord(true);
        textWall.setFocusable(false);

        countDown.setFont(lowerFont);
        countDown.setHorizontalAlignment(SwingConstants.LEFT);
        countDown.setForeground(fontColor);
        extraField.setFont(lowerFont);
        extraField.setHorizontalAlignment(SwingConstants.RIGHT);
        extraField.setForeground(fontColor);

        lowerPanel.setBackground(Color.BLACK);
        container.setBackground(Color.BLACK);

        countDown.setText(" 15:00.0");
        extraField.setText("  ");
    }


//    @Override
//    public void startTimer(int seconds) {
//        timerSeconds = seconds*10;
//        timerSecondsLast = seconds*10;
//        timer.start();
//    }
//
//    @Override
//    public void stopTimer() {
//        timer.stop();
//        timerSeconds = 0;
//        timerSecondsLast = 0;
//    }
//
//    @Override
//    public void pauseTimer(Boolean pause) {
//        if(pause) {
//            timer.stop();
//        } else {
//            timer.start();
//        }
//    }

    @Override
    public void setWall(final String message) {
        final Runnable runnable = new Runnable() {
            @Override
            public void run() {
                textWall.setText(message);
            }
        };

        SwingUtilities.invokeLater(runnable);
    }

    @Override
    public void setExtra(final String text) {
        final Runnable runnable = new Runnable() {
            @Override
            public void run() {
                extraField.setText(text + " ");
            }
        };

        SwingUtilities.invokeLater(runnable);
    }

    @Override
    public void setCountDown(final int tseconds) {
        final Runnable runnable = new Runnable() {
            @Override
            public void run() {
                setCountDownText(tseconds);
            }
        };

        SwingUtilities.invokeLater(runnable);
    }
    
    @Override
    public void showCountDown(final Boolean show) {
        final Runnable runnable = new Runnable() {
            @Override
            public void run() {
                countDown.setVisible(show);
            }
        };

        SwingUtilities.invokeLater(runnable);
        
    }

    @Override
    public void addListener(final GuiEventListener listener) {
        listenerList.add(listener);
    }



}
